from newspy.models import Publication
from newspy.newsorg.client import NewsorgClient, NewsorgEndpoint
from newspy.shared.http_client import HttpClient


class Newspy:
    def __init__(self, newsorg_key: str) -> None:
        self._newsorg_key = newsorg_key

    def publications(self) -> list[Publication]:
        newsorg_client = NewsorgClient(
            http_client=HttpClient(), api_key=self._newsorg_key
        )

        return newsorg_client.publications(endpoint=NewsorgEndpoint.TOP_HEADLINES)



# businesslive.co.za
# ft.com
# reuters.com
# bbc.com
# theguardian.com
# thetimes.co.uk
# wsj.com
# bloomberg.com
# economist.com
# nytimes.com

def main():
    pass


if __name__ == "__main__":
    main()



from dataclasses import dataclass
from enum import Enum

from newspy.models import Publication, Publisher, Source
from newspy.shared import utils
from newspy.shared.utils import slugify


class NewsorgCategory(str, Enum):
    BUSINESS = "business"
    ENTERTAINMENT = "entertainment"
    GENERAL = "general"
    HEALTH = "health"
    SCIENCE = "science"
    SPORTS = "sports"
    TECHNOLOGY = "technology"


@dataclass
class NewsorgArticlesReq:
    """Newsorg API article requests"""

    url: str
    params: dict | None = None


@dataclass
class NewsorgArticleSourceRes:
    """Newsorg API article source response"""

    status: str
    sources: list[Source]

    def __post_init__(self):
        if isinstance(self.sources, list):
            self.sources = [
                Source(**source) if isinstance(source, dict) else source
                for source in self.sources
            ]


@dataclass
class NewsorgArticle:
    """Newsorg API articles"""

    source: Source
    author: str | None
    title: str
    description: str
    url: str
    urlToImage: str
    publishedAt: str
    content: str

    def __post_init__(self):
        if isinstance(self.source, dict):
            self.source = Source(**self.source)

    def to_publication(self) -> Publication:
        publisher = Publisher(
            id=self.source.id,
            name=self.source.name,
        )

        return Publication(
            slug=f"{slugify(self.source.name)}-{slugify(self.title)}",
            url=self.url,
            url_to_image=self.urlToImage,
            title=self.title,
            abstract=self.description,
            author=self.author,
            publisher=publisher,
            published=utils.to_datetime(self.publishedAt),
        )


@dataclass
class NewsorgArticlesRes:
    """Response from the Newsorg API"""

    status: str
    totalResults: int
    articles: list[NewsorgArticle]

    def __post_init__(self):
        if isinstance(self.articles, list):
            self.articles = [
                NewsorgArticle(**article) if isinstance(article, dict) else article
                for article in self.articles
            ]



import logging
from dataclasses import dataclass

from slugify import slugify

from newspy.models import Publication, Publisher
from newspy.shared import utils

logger = logging.getLogger(__name__)


@dataclass
class RssArticleMediaContent:
    height: str
    width: str
    url: str
    medium: str | None


@dataclass
class RssArticleContent:
    base: str
    value: str


@dataclass
class RssArticle:
    id: str
    title: str
    links: list[dict]
    media_content: list[RssArticleMediaContent]
    content: list[RssArticleContent]
    published: str

    def to_publication(self, publisher: Publisher) -> Publication:
        try:
            return Publication(
                slug=f"{slugify(publisher.name)}-{slugify(self.title)}",
                url=self.id,
                url_to_image=self.media_content[0].url,
                title=self.title,
                abstract=self.content[0].value,
                author=None,
                publisher=publisher,
                published=utils.to_datetime(self.published),
            )
        except ValueError:
            logger.warning(
                msg=f"Failed to parse the article with title: {self.title}, id: {self.id}, publisher: {publisher.name}"
            )
            pass



import logging
from enum import Enum

from newspy.models import Language, Country
from newspy.newsorg.models import (
    Publication,
    NewsorgArticlesRes,
    NewsorgCategory,
    NewsorgArticleSourceRes,
    Source,
)
from newspy.shared.exceptions import NewspyException
from newspy.shared.http_client import HttpClient, HttpMethod

logger = logging.getLogger(__name__)

BASE_URL = "https://newsapi.org/v2"


class NewsorgEndpoint(str, Enum):
    EVERYTHING = "EVERYTHING"
    TOP_HEADLINES = "TOP_HEADLINES"
    SOURCES = "SOURCES"


def create_url(endpoint: NewsorgEndpoint) -> str:
    match endpoint:
        case NewsorgEndpoint.EVERYTHING:
            return f"{BASE_URL}/everything"
        case NewsorgEndpoint.TOP_HEADLINES:
            return f"{BASE_URL}/top-headlines"
        case NewsorgEndpoint.SOURCES:
            return f"{BASE_URL}/top-headlines/sources"
        case _:
            raise NewspyException(
                msg=f"The endpoint has to be one of the following values: 'EVERYTHING' or 'TOP_HEADLINES' or 'SOURCES'",
            )


class NewsorgClient:
    def __init__(self, http_client: HttpClient, api_key: str) -> None:
        self._http_client = http_client
        self._api_key = api_key

    def publications(
        self,
        endpoint: NewsorgEndpoint,
        search_text: str | None = None,
        category: NewsorgCategory | None = None,
        country: Country | None = None,
        sources: list[Source] | None = None,
    ) -> list[Publication]:
        if category and sources:
            raise NewspyException(
                msg="Choose either the category and sources attributes. Not both.",
            )

        params = {"apiKey": self._api_key, "pageSize": 100, "page": 1}

        if search_text:
            params["q"] = search_text
        if category and endpoint == NewsorgEndpoint.TOP_HEADLINES:
            params["category"] = category.value
        if country and endpoint == NewsorgEndpoint.TOP_HEADLINES:
            params["country"] = country.value
        if sources and len(sources) > 0:
            params["sources"] = ",".join([source.id for source in sources])

        resp_json = self._http_client.send(
            method=HttpMethod.GET, url=create_url(endpoint=endpoint), params=params
        )

        try:
            article_res = NewsorgArticlesRes(**resp_json)
        except TypeError as type_error:
            raise NewspyException(
                msg=f"Failed to validate the News Org articles response json: {resp_json}",
                reason=str(type_error),
            )

        return [article.to_publication() for article in article_res.articles]

    def sources(
        self,
        endpoint=NewsorgEndpoint.SOURCES,
        category: NewsorgCategory | None = None,
        language: Language | None = None,
        country: Country | None = None,
    ) -> list[Source]:
        params = {"apiKey": self._api_key}

        if category:
            params["category"] = category.value
        if language:
            params["language"] = language.value
        if country:
            params["country"] = country.value

        resp_json = self._http_client.send(
            method=HttpMethod.GET, url=create_url(endpoint=endpoint), params=params
        )

        try:
            return NewsorgArticleSourceRes(**resp_json).sources
        except TypeError as type_error:
            raise NewspyException(
                msg=f"Failed to validate the News Org sources response json: {resp_json}",
                reason=str(type_error),
            )
